package com.lmax.api.internal.order;

import com.lmax.api.FixedPointNumber;
import com.lmax.api.TimeInForce;
import com.lmax.api.internal.Request;
import com.lmax.api.internal.xml.StructuredWriter;
import com.lmax.api.order.OrderSpecification;

public abstract class AbstractOrderSpecification implements Request, OrderSpecification
{
    private long instrumentId;
    private String instructionId;
    private FixedPointNumber quantity;
    private FixedPointNumber stopLossPriceOffset;
    private FixedPointNumber stopProfitPriceOffset;

    private TimeInForce timeInForce;
    private static final String INSTRUCTION_ID_TAG_NAME = "instructionId";
    private static final String STOP_LOSS_OFFSET_TAG_NAME = "stopLossOffset";
    private static final String STOP_PROFIT_OFFSET_TAG_NAME = "stopProfitOffset";

    protected AbstractOrderSpecification(long instrumentId, String instructionId, final TimeInForce timeInForce, FixedPointNumber quantity, final FixedPointNumber stopLossPriceOffset,
                                         final FixedPointNumber stopProfitPriceOffset)
    {
        setInstrumentId(instrumentId);
        setInstructionId(instructionId);
        setTimeInForce(timeInForce);
        setQuantity(quantity);
        setStopLossPriceOffset(stopLossPriceOffset);
        setStopProfitPriceOffset(stopProfitPriceOffset);
    }

    @Override
    public void setInstructionId(final String instructionId)
    {
        this.instructionId = instructionId;
    }

    @Override
    public void setInstrumentId(final long instrumentId)
    {
        this.instrumentId = instrumentId;
    }

    @Override
    public void setQuantity(final FixedPointNumber quantity)
    {
        this.quantity = quantity;
    }

    @Override
    public void setStopLossPriceOffset(final FixedPointNumber stopLossPriceOffset)
    {
        this.stopLossPriceOffset = stopLossPriceOffset;
    }

    @Override
    public void setStopProfitPriceOffset(final FixedPointNumber stopProfitPriceOffset)
    {
        this.stopProfitPriceOffset = stopProfitPriceOffset;
    }

    @Override
    public void setTimeInForce(final TimeInForce timeInForce)
    {
        if (timeInForce == TimeInForce.UNKNOWN)
        {
            throw new IllegalArgumentException("Cannot specify UNKNOWN as time in force.");
        }

        this.timeInForce = timeInForce;
    }

    /**
     * Convert this request to XML.
     *
     * @return The xml representation of this request.
     */
    public void writeTo(StructuredWriter writer)
    {
        writer.startElement("req").startElement("body").
                   startElement("order").
                       value("instrumentId", instrumentId).
                       valueOrNone(INSTRUCTION_ID_TAG_NAME, instructionId).
                       valueOrNone("price", getPrice()).
                       value("quantity", quantity).
                       value("timeInForce", timeInForce.asCamelCase()).
                       valueOrNone(STOP_LOSS_OFFSET_TAG_NAME, stopLossPriceOffset).
                       valueOrNone(STOP_PROFIT_OFFSET_TAG_NAME, stopProfitPriceOffset).
                   endElement("order").
               endElement("body").endElement("req");
    }
    
    protected abstract FixedPointNumber getPrice();

    /**
     * The place order URI.
     *
     * @return The place order URI.
     */
    @Override
    public String getUri()
    {
        return "/secure/trade/placeOrder";
    }

    @Override
    public String toString()
    {
        return new StringBuilder()
            .append("instrumentId=").append(instrumentId)
            .append(", instructionId=").append(instructionId)
            .append(", quantity=").append(quantity)
            .append(", stopLossPriceOffset=").append(stopLossPriceOffset)
            .append(", stopProfitPriceOffset=").append(stopProfitPriceOffset)
            .append(", timeInForce=").append(timeInForce.asCamelCase())
            .toString();
    }
}
